////////////////////////////////////////////////////
//
////////////////////////////////////////////////////

#ifndef _UOSDEVICEC_H_
#define _UOSDEVICEC_H_

#ifdef __cplusplus
extern "C" {
#endif

#pragma pack(4)

#define VERSION "1.0.0.17"

//序列号版本定义
#define V1_SER_VER (0) //一期序列号版本
#define V2_SER_VER (1) //二期序列号版本

enum {
  ECC_SUCC = (unsigned int)0, //ECC相关处理正确
  ECC_ERR_PRODUCT,            //产品版本错误/不匹配
  ECC_ERR_VER,                //版本错误
  ECC_ERR_INIT,               //初始化参数错误
  ECC_ERR_PRIKEY,             //私钥参数错误
  ECC_ERR_PUBKEY,             //公钥参数错误
  ECC_ERR_CODE,               //编解码错误
  ECC_ERR_SUM,                //SUM校验错误
  ECC_ERR_VERIFY,             //校验失败
  ECC_ERR_DATALEN,            //数据长度错误
  ECC_ERR_UNKNOWEN,           //未知错误
};

//======================这块代码是将库代码中的内存管理移到调用者管理时加入的。
//签名数据长度（序列号中用来表示加签数据的长度）
#define V1_SERIAL_STR_LEN (16)
#define V2_SERIAL_STR_LEN (25)

//二期签名数据位数（基于BASE32编解码）
#define V2_SERIAL_DATA_BITSIZE (V2_SERIAL_STR_LEN * 5)
#define V2_SERAIL_DATA_VER_BITSIZE (4)
#define V2_SERIAL_DATA_RAND_BITSIZE (2)
#define V2_SERIAL_DATA_PRODUCT_BITSIZE (28)
#define V2_SERIAL_DATA_OEMFLAG_BITSIZE (1)
#define V2_SERIAL_DATA_AUTHMODE_BITSIZE (3)
#define V2_SERIAL_DATA_ECCVER_BITSIZE (4)
#define V2_SERIAL_DATA_SIGN_BITSIZE (47)
#define V2_SERIAL_DATA_HASH_BITSIZE (32)
#define V2_SERIAL_DATA_SUM_BITSIZE (4)

#define V2_SERIAL_DATA_INFO_BITSIZE                                            \
  (V2_SERIAL_DATA_BITSIZE - V2_SERAIL_DATA_VER_BITSIZE -                       \
   V2_SERIAL_DATA_RAND_BITSIZE)

#if (0 == (V2_SERIAL_DATA_BITSIZE % 8))
#define V2_SERIAL_DATA_BYTESIZE (V2_SERIAL_DATA_BITSIZE / 8)
#else
#define V2_SERIAL_DATA_BYTESIZE ((V2_SERIAL_DATA_BITSIZE / 8) + 1)
#endif

#if (0 == (V2_SERIAL_DATA_INFO_BITSIZE % 8))
#define V2_SERIAL_DATA_INFO_BYTESIZE (V2_SERIAL_DATA_INFO_BITSIZE / 8)
#else
#define V2_SERIAL_DATA_INFO_BYTESIZE ((V2_SERIAL_DATA_INFO_BITSIZE / 8) + 1)
#endif

#if (0 == (V2_SERIAL_DATA_SIGN_BITSIZE % 8))
#define V2_SERIAL_DATA_SIGN_BYTESIZE (V2_SERIAL_DATA_SIGN_BITSIZE / 8)
#else
#define V2_SERIAL_DATA_SIGN_BYTESIZE ((V2_SERIAL_DATA_SIGN_BITSIZE / 8) + 1)
#endif

#if (0 == (V2_SERIAL_DATA_HASH_BITSIZE % 8))
#define V2_SERIAL_DATA_HASH_BYTESIZE (V2_SERIAL_DATA_HASH_BITSIZE / 8)
#else
#define V2_SERIAL_DATA_HASH_BYTESIZE ((V2_SERIAL_DATA_HASH_BITSIZE / 8) + 1)
#endif

#define SER_INST_KEY_MAXLEN 512
#define SER_INST_SIGN_MAXLEN 100

#define SER_VER_NUM 16
#define ECC_NUM_EACH_SER_VER 16

//序列号示例ECC曲线部分参数
typedef struct {
  char aucPubKeyStr[SER_INST_KEY_MAXLEN];
  char aucPriKeyStr[SER_INST_KEY_MAXLEN];
  char aucToSignStr[SER_INST_SIGN_MAXLEN];
  unsigned char ucEccVer;
  void *pEcc;//库内部使用时需要强转成(curve *)
} serEccParam;

//序列号实例参数部分
typedef struct {
  serEccParam eccInst[ECC_NUM_EACH_SER_VER];
} serialInst;

//序列号实例
typedef struct {
  serialInst serInst[SER_VER_NUM];
} serial;
//====================END=============================================

typedef struct {
    unsigned char arrProductID[4]; // 序列号中的产品ID(该参数为28bit，格式是数组，不考虑字节序问题)
    unsigned char oemFlag; // oem标志。0:非oem,1:oem
    unsigned char authMode; // 授权方式
} SerialInfo;


// ECC Api definition
/**************************ECC实例API接口说明*************************
使用一个ECC实例时，步骤如下
1) ecc_init(),分配一个ecc曲线实例
2) ecc_init_with_param,为该ecc实例参数赋值
3) ecc_init_pubkey/ecc_init_prikey,为该ecc实例设置公钥/私钥
4) 随后使用对应的接口进行操作
   a) ecc_sign_str,用于加签。在签名之前必须设置私钥
   b) ecc_verify_str,用于验签。在验签之前必须设置公钥
   c) ecc_encrypt_str,用于加密字符串。在加密之前必须设置公钥
   d) ecc_decrypt_to_str,用于把加密后的数据解密为字符串。在解密之前必须设置私钥
5) ecc_uninit(),释放ecc曲线实例
NOTE：一个ECC实例，曲线参数、公钥、私钥是唯一的；
     如果曲线的参数一样，但是公钥、私钥不一致；应当申请不同的ecc实例
     ecc_generate_private---可用在ecc_init_with_param设置参数之后，用来生成ecc曲线私钥
     ecc_generate_public----可用在ecc_init_with_param设置参数之后，用来生成ecc曲线公钥
*********************************************************************/
//获取一个ecc曲线实例
void *ecc_init();

//释放一个ecc曲线实例
void ecc_uninit(void *pEcc // In param:需要释放的的ECC曲线实例指针
);

// Init a new Ecc inst with special parameter
// 0:OK
// others:error
unsigned int ecc_init_with_param(
    void *pEccCurveInst, // In Param:ECC曲线实例缓冲区
    char *
        eccAHexStr, // In
                    // Param:ECC曲线的a参数，ASCII字符串表示的16进制数据，头部是否包含0x都可，入参
    char *
        eccBHexStr, // In
                    // Param:ECC曲线的b参数，ASCII字符串表示的16进制数据，头部是否包含0x都可
    char *
        eccPHexStr, // In
                    // Param:ECC曲线的p参数，ASCII字符串表示的16进制数据，头部是否包含0x都可
    char *
        eccGxHexStr, // In
                     // Param:ECC曲线的基点G的坐标x，ASCII字符串表示的16进制数据，头部是否包含0x都可
    char *
        eccGyHexStr, // In
                     // Param:ECC曲线的基点G的坐标y，ASCII字符串表示的16进制数据，头部是否包含0x都可
    char *
        eccOrderHexStr, // In
                        // Param:ECC曲线的阶，ASCII字符串表示的16进制数据，头部是否包含0x都可
    unsigned int signOutStrLen // In Param:签名输出的字符串的长度
);

//为一个ecc曲线实例初始化私钥
unsigned int ecc_init_prikey(void *pEcc, // In param:ECC曲线实例指针
                             char *priKeyStr // In param:ECC曲线实例私钥字符串
);

//为一个ecc曲线实例初始化公钥
unsigned int ecc_init_pubkey(void *pEcc, // In param:ECC曲线实例指针
                             char *pubKeyStr // In param:ECC曲线实例公钥字符串
);

// 对字符串进行加签
// 0: ok
// others: error
unsigned int ecc_sign_str(void *pEccCurveInst, // In Param:ECC曲线实例缓冲区
                          char *toSignData, // In Param:进行加签的字符串
                          char *outBuf // In Param:签名后输出数据的缓冲区
);

// 对字符串进行验签
// 0: ok
// others: error
unsigned int ecc_verify_str(void *pEccCurveInst, // In Param:ECC曲线实例缓冲区
                            char *srcCode, // In Param:进行加签的源字符串
                            char *toVerifyStr // In Param:进行验签的字符串
);

//加密字符串
// 0: OK
// others: error
unsigned int
ecc_encrypt_str(void *pEccCurveInst, // In param:用来进行加密的ECC曲线实例
                char *toEncryStr, // In param:用来进行加密的字符串
                unsigned char *outBuf, // In param:输出加密数据的缓冲区
                unsigned int bufSize, // In param:输出加密数据的缓冲区的长度
                unsigned int *outEncryptDataLen // In param:输出加密后的数据长度
);

//解密字符串
// 0: OK
// others: error
unsigned int
ecc_decrypt_to_str(void *pEccCurveInst, // In param:用来进行解密的ECC曲线实例
                   unsigned char *data, // In param:用来进行解密的数据
                   char *outBuf, // In param:输出解密数据的缓冲区
                   unsigned int bufSize // In param:解密数据缓冲区的长度
);

// 生成私钥
// 0: ok
// others: error
unsigned int
ecc_generate_private(void *pEccCurveInst, // In Param:ECC曲线实例缓冲区
                     char *priKeyBuf,     // In Param:输出的私钥缓冲区
                     int bufSize          // In Param:私钥缓冲区的长度
);

// 生成公钥
// 0: ok
// others: error
unsigned int
ecc_generate_public(void *pEccCurveInst, // In Param:ECC曲线实例缓冲区
                    char *priKeyStr,     // In Param:私钥
                    char *pubKeyBuf,     // In Param:输出的公钥缓冲区
                    int bufSize          // In Param:公钥缓冲区长度
);
// ECC serial NO. Api definition
/**************************ECC序列号实例API接口说明********************
使用一个ECC序列号实例时，步骤如下
1) ecc_ser_init(),分配一个ecc序列号实例
2) ecc_ser_init_with_param,为该ecc序列号实例参数赋值
3) ecc_ser_init_pubkey/ecc_ser_init_prikey,为该序列号实例设置公钥/私钥
4) 随后使用对应的接口进行操作
   a) ecc_gen_ser,用于生成序列号。在生成序列号之前必须设置私钥
   b) ecc_verify_ser,用于验证序列号。在验证序列号之前必须设置公钥
5) ecc_ser_uninit(),释放ecc序列号实例。
*********************************************************************/
//获取一个ecc序列号实例
void ecc_ser_init();
void ecc_ser_init_v2(void *gpSer);

//释放一个ecc序列号实例
void ecc_ser_uninit();
void ecc_ser_uninit_v2(void *gpSer);

// Init a new serial inst with special parameter
// 0:OK
// others:error
unsigned int
ecc_ser_init_with_param(unsigned char serVer, // In Param:序列号版本
                        unsigned char eccVer, // In Param:ecc参数版本
                        char *toSignStr // In Param:进行加签的字符串
);

unsigned int
ecc_ser_init_with_param_v2(void *gpSer,unsigned char serVer, // In Param:序列号版本
                        unsigned char eccVer, // In Param:ecc参数版本
                        char *toSignStr // In Param:进行加签的字符串
);

// 对序列号实例配置私钥
// 0: ok
// others: error
unsigned int
ecc_ser_init_prikey(unsigned char serVer, // In Param:序列号版本
                    unsigned char eccVer, // In Param:ECC参数/算法版本
                    char *pPriKeyStr      // In Param:私钥
);
unsigned int
ecc_ser_init_prikey_v2(void *gpSer,unsigned char serVer, // In Param:序列号版本
                    unsigned char eccVer, // In Param:ECC参数/算法版本
                    char *pPriKeyStr      // In Param:私钥
);


// 对序列号实例配置公钥
// 0: ok
// others: error
unsigned int
ecc_ser_init_pubkey(unsigned char serVer, // In Param:序列号版本
                    unsigned char eccVer, // In Param:ECC参数/算法版本
                    char *pPubKeyStr      // In Param:公钥
);
unsigned int
ecc_ser_init_pubkey_v2(void *gpSer,unsigned char serVer, // In Param:序列号版本
                    unsigned char eccVer, // In Param:ECC参数/算法版本
                    char *pPubKeyStr      // In Param:公钥
);


//验证序列号
// 0: OK
// others: error
unsigned int ecc_verify_ser(
    unsigned char
        arrProductID[4], // In Param:product
                         // ID(该参数为28bit，格式是数组，不考虑字节序问题)
    char *serialNoStr // In Param:需要进行验证的序列号
);
unsigned int ecc_verify_ser_v2(void *gpSer,
    unsigned char
        arrProductID[4], // In Param:product
                         // ID(该参数为28bit，格式是数组，不考虑字节序问题)
    char *serialNoStr // In Param:需要进行验证的序列号
);


//生成序列号
// 0: OK
// others: error
unsigned int ecc_gen_ser(
    unsigned char serVer, // In Param:序列号版本
    unsigned char eccVer, // In Param:使用的ECC曲线/参数版本
    unsigned char
        arrProductID[4], // In Param:product
                         // ID(该参数为28bit，格式是数组，不考虑字节序问题)
    unsigned char oemFlag, // In Param:oem标记，0-非OEM，1-OEM
    unsigned char authMode, // In Param:授权模式
    char *outSerNoStr       // In Param:输出的序列号缓冲区
);
unsigned int ecc_gen_ser_all(void *gpSer,
    unsigned char serVer, // In Param:序列号版本
    unsigned char eccVer, // In Param:使用的ECC曲线/参数版本
    unsigned char
        arrProductID[4], // In Param:product
                         // ID(该参数为28bit，格式是数组，不考虑字节序问题)
    unsigned char oemFlag, // In Param:oem标记，0-非OEM，1-OEM
    unsigned char authMode, // In Param:授权模式
    char *outSerNoStr       // In Param:输出的序列号缓冲区
);

//获取序列号授权模式
// 255: 无效授权模式/未正确获取授权模式
// others: 获取的授权模式
unsigned char
ecc_ser_get_authmode(char *serialNoStr // In Param:需要获取授权模式的序列号
);

unsigned int
ecc_get_ser_info(char *serialStr, // In Param:需要获取信息的序列号
    SerialInfo *serialInfo // Out Param:输出序列号信息
);

#pragma pack()

#ifdef __cplusplus
}
#endif
#endif //#ifndef _UOSDEVICEC_H_
